import io
import json
import os
import tempfile

import requests
import responses

import marcx
import pymarc
from siskin.utils import (SetEncoder, URLCache, dictcheck, get_task_import_cache, load_set, marc_build_imprint,
                          marc_clean_record, marc_clean_subfields, nwise, random_string, scrape_html_listing, xmlstream)

def test_set_encoder_dumps():
    assert json.dumps({'x': {0, 1, 2}}, cls=SetEncoder) == '{"x": [0, 1, 2]}'


def test_dictcheck():
    assert dictcheck({"name": "x"}, contains=["name"]) is True
    assert dictcheck({"name": "x"}, contains=["name"], absent=["somekey"]) is True
    assert dictcheck({"name": "x", "somekey": 123}, contains=["name"], absent=["somekey"]) is False
    assert dictcheck({"somekey": None}, absent=["somekey"]) is True
    assert dictcheck({}, absent=["somekey"]) is True


def test_nwise():
    assert list(nwise(range(4))) == [(0, 1), (2, 3)]
    assert list(nwise(range(4), n=2)) == [(0, 1), (2, 3)]
    assert list(nwise(range(4), n=3)) == [(0, 1, 2), (3, )]
    assert list(nwise(range(4), n=10)) == [(0, 1, 2, 3)]
    assert list(nwise([], n=10)) == []


def test_random_string():
    assert len(random_string()) == 16
    assert len(random_string(length=1000)) == 1000
    assert len(random_string(length=-1)) == 0
    assert random_string(length=-100) == ''
    for char in ' \t\n.:,;#~+-?=[]()/&%$"!':
        assert char not in random_string()


def test_get_task_import_cache():
    mapping, path = get_task_import_cache()
    assert os.path.exists(path)
    assert isinstance(mapping, dict)


def test_load_set():
    assert load_set(io.StringIO(u"")) == set()
    assert load_set(io.StringIO(u"1\n1\n1\n")) == {"1"}
    assert load_set(io.StringIO(u"1\n    \n2\n")) == {"1", "2"}
    assert load_set(io.StringIO(u"1\n2\n3\n")) == {"1", "2", "3"}
    assert load_set(io.StringIO(u"1\n2\n3\n"), func=int) == {1, 2, 3}

    with tempfile.NamedTemporaryFile(delete=False) as tf:
        tf.write(b"1\n2\n")
    with open(tf.name) as handle:
        assert load_set(handle) == {"1", "2"}
    assert load_set(tf.name) == {"1", "2"}
    os.remove(tf.name)


def test_get_cache_file(tmpdir):
    cache = URLCache(directory=str(tmpdir))
    fn = cache.get_cache_file("http://x.com")
    assert fn.startswith(str(tmpdir))


@responses.activate
def test_scrape_html_listing():
    responses.add(responses.GET, 'http://fake.com/1', body='<html></html>', status=200)
    resp = requests.get('http://fake.com/1')
    assert scrape_html_listing('http://fake.com/1') == []

    body = """
    <!doctype html><title>Index of /archlinux/iso/2018.12.01</title><h1>Index
    of /archlinux/iso/2018.12.01</h1><table><tr><th valign=top><img
    src=/icons/blank.gif alt=[ICO]><th><a href="?C=N;O=D">Name</a><th><a
    href="?C=M;O=A">Last modified</a><th><a href="?C=S;O=A">Size</a><th><a
    href="?C=D;O=A">Description</a><tr><th colspan=5><hr><tr><td valign=top><img
    src=/icons/back.gif alt=[PARENTDIR]><td><a href=/archlinux/iso/>Parent
    Directory</a><td>&nbsp;<td align=right>-<td>&nbsp;<tr><td valign=top><img
    src=/icons/folder.gif alt=[DIR]><td><a href=arch/>arch/</a><td
    align=right>2018-12-01 11:16<td align=right>-<td>&nbsp;<tr><td valign=top><img
    src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso>archlinux-2018.12.01-x86_64.iso</a><td
    align=right>2018-12-01 11:17<td align=right>588M<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso.sig>archlinux-2018.12.01-x86_64.iso.sig</a><td
    align=right>2018-12-01 11:18<td align=right>310<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso.torrent>archlinux-2018.12.01-x86_64.iso.torrent</a><td
    align=right>2018-12-01 11:18<td align=right>38K<td>&nbsp;<tr><td
    valign=top><img src=/icons/compressed.gif alt="[   ]"><td><a
    href=archlinux-bootstrap-2018.12.01-x86_64.tar.gz>archlinux-bootstrap-2018.12.01-x86_64.tar.gz</a><td
    align=right>2018-12-01 11:18<td align=right>141M<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig>archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig</a><td
    align=right>2018-12-01 11:18<td align=right>310<td>&nbsp;<tr><td
    valign=top><img src=/icons/text.gif alt=[TXT]><td><a
    href=md5sums.txt>md5sums.txt</a><td align=right>2018-12-01 11:18<td
    align=right>145<td>&nbsp;<tr><td valign=top><img src=/icons/text.gif
    alt=[TXT]><td><a href=sha1sums.txt>sha1sums.txt</a><td align=right>2018-12-01
    11:18<td align=right>161<td>&nbsp;<tr><th
    colspan=5><hr></table><address>Apache/2.4.25 (Debian) Server at
    ftp.halifax.rwth-aachen.de Port 80</address>
    """

    expected = [
        'http://fake.com/archlinux-2018.12.01-x86_64.iso',
        'http://fake.com/archlinux-2018.12.01-x86_64.iso.sig',
        'http://fake.com/archlinux-2018.12.01-x86_64.iso.torr',
        'http://fake.com/archlinux-bootstrap-2018.12.01-x86_64.tar.gz',
        'http://fake.com/archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig',
        'http://fake.com/md5sums.txt',
        'http://fake.com/sha1sums.txt',
    ]

    responses.add(responses.GET, 'http://fake.com/1', body=body, status=200)
    resp = requests.get('http://fake.com/1')
    assert scrape_html_listing('http://fake.com/1') == expected


def test_xmlstream():
    with tempfile.NamedTemporaryFile('w', delete=False) as handle:
        handle.write("""<a><b>C</b><b>C</b></a>""")

    filename = handle.name
    assert [v for v in xmlstream(filename, "b")] == [b'<b>C</b>', b'<b>C</b>']
    os.remove(filename)


def test_marc_clean_subfields():
    record = marcx.Record()
    record.add("001", data="1234")
    record.add("245", a="", b="ok")

    assert record.strict == True

    # Behind the scenes, marcx will not add empty subfield values
    # (https://git.io/fjIWU).
    assert marc_clean_subfields(record["245"], inplace=False) == ['b', 'ok']
    assert record["245"].subfields == ['b', 'ok']

    assert marc_clean_subfields(record["245"], inplace=True) is None
    assert record["245"].subfields == ['b', 'ok']

    # Test pymarc record.
    record = pymarc.Record()
    record.add_field(pymarc.Field(tag='001', data='1234'))
    record.add_field(pymarc.Field(tag='245', indicators=['0', '1'], subfields=['a', '', 'b', 'ok']))

    assert len(record.get_fields()) == 2

    assert marc_clean_subfields(record["245"], inplace=False) == ['b', 'ok']
    assert record["245"].subfields == ['a', '', 'b', 'ok']
    assert marc_clean_subfields(record["245"], inplace=True) is None
    assert record["245"].subfields == ['b', 'ok']


def test_marc_clean_record():
    record = pymarc.Record()
    record.add_field(pymarc.Field(tag='001', data='1234'))
    record.add_field(pymarc.Field(tag='245', indicators=['0', '1'], subfields=['a', '', 'b', 'ok']))

    assert len(record.get_fields()) == 2

    assert record["245"].subfields == ['a', '', 'b', 'ok']
    marc_clean_record(record)
    assert record["245"].subfields == ['b', 'ok']


def test_marc_build_imprint():
    assert marc_build_imprint() == ['a', '', 'b', '', 'c', '']
    assert marc_build_imprint(place="A") == ['a', 'A', 'b', '', 'c', '']
    assert marc_build_imprint(publisher="B") == ['a', '', 'b', 'B', 'c', '']
    assert marc_build_imprint(year="C") == ['a', '', 'b', '', 'c', 'C']
    assert marc_build_imprint(place="A", publisher="B") == ['a', 'A : ', 'b', 'B', 'c', '']
    assert marc_build_imprint(place="A", year="C") == ['a', 'A', 'b', ', ', 'c', 'C']
    assert marc_build_imprint(publisher="B", year="C") == ['a', '', 'b', 'B, ', 'c', 'C']
    assert marc_build_imprint(place="A", publisher="B", year="C") == ['a', 'A : ', 'b', 'B, ', 'c', 'C']



#!/usr/bin/env python
# coding: utf-8
"""
Custom conversion for Gutenberg RDF, refs. #10875.

Usage:

    $ wget http://www.gutenberg.org/cache/epub/feeds/rdf-files.tar.zip
    $ unzip -p rdf-files.tar.zip | tar -xO | python 1_marcbinary.py -

"""

import base64
import fileinput
import io
import json
import re
import sys

import xmltodict

import marcx

inputfile = fileinput.input()

# Workaround, so we can stream to stdout as well.
outputfile = sys.stdout

if "-" not in sys.argv[1:]:
    outputfile = open("1_output.mrc", "wb")

langmap = {
    "da": "dan",
    "de": "ger",
    "el": "ell",
    "en": "eng",
    "eo": "epo",
    "es": "spa",
    "fi": "fin",
    "fr": "fre",
    "it": "ita",
    "la": "lat",
    "nl": "nld",
    "no": "nor",
    "pt": "por",
    "ro": "rom",
    "ru": "rus",
    "sv": "swe",
    "ja": "jpn",
    "he": "heb",
    "hu": "hun",
    "pl": "pol",
    "cy": "cym",
    "cs": "ces",
    "yi": "yid",
    "bg": "bul",
    "is": "isl",
    "ga": "gle",
    "sr": "srp",
    "af": "afr",
    "iu": "iku",
    "fy": "fry",
    "br": "bre",
    "arp": "arp",
    "lt": "lit",
    "tl": "tgl",
    "ca": "cat",
    "myn": "myn",
    "fur": "fur",
    "enm": "enm",
    "oc": "oci",
    "gla": "gla",
    "rmr": "rmr",
    "mi": "mri",
    "ilo": "ilo",
    "ia": "ina",
    "nap": "nap",
    "ceb": "ceb",
    "gl": "glg",
    "nah": "nah",
    "sl": "slv",
    "te": "tel",
    "oji": "oji",
    "ar": "ara",
    "et": "est",
    "fa": "fas",
    "sa": "san",
    "zh": "zho"
}

record = []
in_record = False
counter = 0

for line in inputfile:
    line = line.strip()

    f001 = ""
    f007 = "sr"
    f010a = ""
    f020a = ""
    f041a = ""
    f100a = ""
    f245a = ""
    f250a = ""
    f260b = ""
    f260c = ""
    f300 = ""
    f300a = ""
    f300b = ""
    f300c = ""
    f490a = ""
    f500a = ""
    f508a = ""
    f520a = ""
    f546a = ""
    f689a = ""
    f700a = ""
    f856u = ""
    files = []
    subjects = []
    authors = []
    leader_6_8 = "nam"

    if line.startswith("<rdf:RDF"):
        in_record = True

    if in_record:
        record.append(line)

    if line.startswith("</rdf:RDF>"):
        record = xmltodict.parse('\n'.join(record),
                                 force_list=("dcterms:hasFormat", "dcterms:subject", "dcterms:creator"))

        for file in record["rdf:RDF"]["pgterms:ebook"].get("dcterms:hasFormat", []):
            files.append(file.get("pgterms:file", {}).get("@rdf:about", ""))

        for file in files:

            if file.endswith(".mp3") or file.endswith(".ogg"):
                leader_6_8 = "cim"
                f007 = "su"

            regexp = re.search("(http:\/\/www\.gutenberg\.org\/ebooks\/\d+)", file)
            if regexp:
                f856u = regexp.group(1)
                break

        try:
            f001 = record["rdf:RDF"]["pgterms:ebook"]["@rdf:about"]
        except (TypeError, KeyError):
            pass

        try:
            f010a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc010"]
        except (TypeError, KeyError):
            pass

        try:
            f020a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc020"]
        except (TypeError, KeyError):
            pass

        try:
            f041a = record["rdf:RDF"]["pgterms:ebook"]["dcterms:language"]["rdf:Description"]["rdf:value"]["#text"]
        except (TypeError, KeyError):
            pass

        try:
            for creator in record["rdf:RDF"]["pgterms:ebook"]["dcterms:creator"]:
                author = creator["pgterms:agent"]["pgterms:name"]
                if author == "Various":
                    author = ""
                authors.append(author)
        except (TypeError, KeyError):
            pass

        try:
            f245a = record["rdf:RDF"]["pgterms:ebook"]["dcterms:title"]
        except (TypeError, KeyError):
            pass

        try:
            f250a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc250"]
        except (TypeError, KeyError):
            pass

        try:
            f260b = record["rdf:RDF"]["pgterms:ebook"]["dcterms:publisher"]
        except (TypeError, KeyError):
            pass

        try:
            f260c = record["rdf:RDF"]["pgterms:ebook"]["dcterms:issued"]["#text"]
        except (TypeError, KeyError):
            pass

        try:
            f300 = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc300"]
        except (TypeError, KeyError):
            pass

        try:
            f490a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc440"]
        except (TypeError, KeyError):
            pass

        try:
            f500a = record["rdf:RDF"]["pgterms:ebook"]["dcterms:description"]
        except (TypeError, KeyError):
            pass

        try:
            f508a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc508"]
        except (TypeError, KeyError):
            pass

        try:
            f520a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc520"]
        except (TypeError, KeyError):
            pass

        try:
            f546a = record["rdf:RDF"]["pgterms:ebook"]["pgterms:marc546"]
        except (TypeError, KeyError):
            pass

        for subject in record["rdf:RDF"]["pgterms:ebook"].get("dcterms:subject", []):
            subjects.append(subject.get("rdf:Description", {}).get("rdf:value", ""))

        marcrecord = marcx.Record(force_utf8=True)
        marcrecord.strict = False

        # Leader
        marcrecord.leader = "     %s  22        4500" % leader_6_8

        # 001
        regexp = re.search("ebooks\/(\d+)", f001)
        if regexp:
            f001 = regexp.group(1)
        else:
            pass
            #print("ID fehlt!")
        marcrecord.add("001", data="finc-1-%s" % f001)

        # 007
        marcrecord.add("007", data=f007)

        # Sprache für 008
        language = f041a
        f041a = langmap.get(f041a, "")
        if f041a == "":
            pass
            #print("Die Sprache %s fehlt in der Lookup-Tabelle!" % language)
        marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % f041a)

        # Übergeordnetes Werk ???
        marcrecord.add("010", a=f010a)

        # ISBN
        marcrecord.add("020", a=f020a)

        # Sprache für 041a
        marcrecord.add("041", a=f041a)

        # 1. Urheber
        if len(authors) > 0:
            f100a = authors[0]
            marcrecord.add("100", a=f100a)

        # Titel
        marcrecord.add("245", a=f245a)

        # Ausgabevermerk ???
        marcrecord.add("250", a=f250a)

        # Erscheinungsjahr (bei Project Gutenberg)
        regexp = re.search("^(\d\d\d\d)", f260c)
        if regexp:
            f260c = regexp.group(1)
        else:
            f260c = ""

        # Erscheinungsvermerk
        marcrecord.add("260", b=f260b, c=f260c)

        # Kollationsangaben
        regexp = re.search("^\$a(.*?)\$b(.*?)\$c(.*?)$", f300)  # $a11 v. :$bill. ;$c24 cm
        if regexp:
            f300a, f300b, f300c = regexp.groups()
        else:
            f300a = f300
        marcrecord.add("300", a=f300a, b=f300b, c=f300c)

        # 440 ??? so in den Rohdaten
        marcrecord.add("490", a=f490a)

        # Fußnote (allgemein)
        marcrecord.add("500", a=f500a)

        # Fußnote (Credits)
        marcrecord.add("508", a=f508a)

        # Zusammenfassung
        marcrecord.add("520", a=f520a)
        marcrecord.add("546", a=f546a)

        # Schlagwörter
        for subject in subjects:
            marcrecord.add("689", a=subject)

        # 2. und mehr Urheber
        if len(authors) > 1:
            for f700a in authors[1:]:
                marcrecord.add("700", a=f700a)

        # URL
        marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

        # Medienform
        marcrecord.add("935", b=["cofz", "sott"])

        # Ansigelung
        marcrecord.add("980", a=f001, b="1", c="Project Gutenberg")

        outputfile.write(marcrecord.as_marc())

        record = []
        in_record = False

    counter += 1

    # if counter == 500000:
    #    break

inputfile.close()

if outputfile == sys.stdout:
    outputfile.flush()
else:
    outputfile.close()



#!/usr/bin/env python3
# coding: utf-8

from __future__ import print_function

import re
import sys

import marcx
import pymarc

copytags = ("100", "105", "120", "130", "150", "174", "200", "245", "246", "250", "260", "300", "335", "336", "337",
            "338", "351", "361", "400", "500", "520", "650", "689", "700", "710", "800")

inputfilename = "156_input.xml"
outputfilename = "156_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")
oldrecords = pymarc.parse_xml_to_array(inputfile)

for i, oldrecord in enumerate(oldrecords, start=1):

    try:
        f245a = oldrecord["245"]["a"]
    except:
        continue

    newrecord = marcx.Record(force_utf8=True)

    # leader
    leader = "     " + oldrecord.leader[5:]
    newrecord.leader = leader

    # 001
    f001 = str(i)
    newrecord.add("001", data="finc-156-" + f001)

    # ISBN
    try:
        f020a = oldrecord["020"]["a"]
    except:
        f020a = ""

    if f020a != "":
        f020a = f020a.replace(" ", "-")
        f020a = f020a.replace(".", "-")
        regexp = re.search("([0-9xX-]{10,17})", f020a)

        if regexp:
            f020a = regexp.group(1)
            f020a = f020a.rstrip("-")
            newrecord.add("020", a=f020a)
        else:
            print(u"Die ISBN %s konnte nicht mittels regulärer Ausdrücke überprüft werden." % f020a, file=sys.stderr)

    # ISSN
    try:
        f022a = oldrecord["022"]["a"]
    except:
        f022a = ""

    if f022a != "":
        f022a = f022a.replace(" ", "-")
        f022a = f022a.replace(".", "-")
        regexp = re.search("([0-9xX-]{9})", f022a)

        if regexp:
            f022a = regexp.group(1)
            f022a = f022a.rstrip("-")
            newrecord.add("022", a=f022a)
        else:
            print(u"Die ISSN %s konnte nicht mittels regulärer Ausdrücke überprüft werden." % f022a, file=sys.stderr)

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # 980
    collections = ["a", f001, "b", "156", "c", "sid-156-col-umweltbibliothek"]
    newrecord.add("980", subfields=collections)

    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



#/usr/bin/env python
# coding: utf-8

import io
import re
import sys

from six.moves import html_parser

import marcx

formatmap = {
    "Buch": {
        "leader": "cam",
        "007": "tu",
        "935b": "druck"
    },
    "DVD": {
        "leader": "ngm",
        "007": "vd",
        "935b": "dvdv",
        "935c": "vide"
    },
    "Blu-ray": {
        "leader": "ngm",
        "007": "vd",
        "935b": "bray",
        "935c": "vide"
    },
    "Videodatei": {
        "leader": "cam",
        "007": "cr",
        "935b": "cofz",
        "935c": "vide"
    },
    "CD": {
        "leader": "  m",
        "007": "c",
        "935b": "cdda"
    },
    "Videokassette": {
        "leader": "cgm",
        "007": "vf",
        "935b": "vika",
        "935c": "vide"
    },
    "Noten": {
        "leader": "nom",
        "007": "zm",
        "935c": "muno"
    },
    "Loseblattsammlung": {
        "leader": "nai",
        "007": "td",
    },
    "Film": {
        "leader": "cam",
        "007": "mu",
        "935b": "sobildtt"
    },
    "Aufsatz": {
        "leader": "naa",
        "007": "tu"
    },
}

parser = html_parser.HTMLParser()


def get_field(tag):
    regexp = re.search('<.*? tag="%s">(.*)$' % tag, field)
    if regexp:
        _field = regexp.group(1)
        _field = parser.unescape(_field)
        return _field
    else:
        return ""


def get_subfield(tag, subfield):
    regexp = re.search('^<datafield.*tag="%s".*><subfield code="%s">(.*?)<\/subfield>' % (tag, subfield), field)
    if regexp:
        _field = regexp.group(1)
        _field = parser.unescape(_field)
        return _field
    else:
        return ""


def get_leader(format="Buch"):
    return "     %s  22        4500" % formatmap[format]["leader"]


def get_field_007(format="Buch"):
    if "007" not in formatmap[format]:
        return ""
    return formatmap[format]["007"]


def get_field_935b(format="Buch"):
    if "935b" not in formatmap[format]:
        return ""
    return formatmap[format]["935b"]


def get_field_935c(format="Buch"):
    if "935c" not in formatmap[format]:
        return ""
    return formatmap[format]["935c"]


# Default input and output.
inputfilename = "151_input.xml"
outputfilename = "151_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "r")
outputfile = io.open(outputfilename, "wb")

records = inputfile.read()
records = records.split("</record>")

for record in records:

    format = ""
    f001 = ""
    f020a = ""
    f100a = ""
    f100e = ""
    f245a = ""
    f260a = ""
    f260b = ""
    f260c = ""
    f300a = ""
    f650a = ""
    f700a = ""
    f700e = ""

    subjects = []
    persons = []
    f100 = []
    f700 = []

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False
    fields = re.split("(</controlfield>|</datafield>)", record)

    for field in fields:

        field = field.replace("\n", "")
        field = field.replace("</datafield>", "")
        field = field.replace("</controlfield>", "")

        # Identfikator
        if f001 == "":
            f001 = get_field("001")

        # ISBN
        if f020a == "":
            f020a = get_subfield("540", "a")

        # Format
        if format == "":
            form = get_field("433")

            regexp1 = re.search("\d\]?\sS\.", form)
            regexp2 = re.search("\d\]?\sSeit", form)
            regexp3 = re.search("\d\]?\sBl", form)
            regexp4 = re.search("\s?Illl?\.", form)
            regexp5 = re.search("[XVI],\s", form)
            regexp6 = re.search("^\d+\s[SsPp]", form)
            regexp7 = re.search("^DVD", form)
            regexp8 = re.search("^Blu.?-[Rr]ay", form)
            regexp9 = re.search("^H[DC] [Cc][Aa][Mm]", form)
            regexp10 = re.search("^HDCAM", form)
            regexp11 = re.search("[Bb]et.?-?[Cc]am", form)
            regexp12 = re.search("CD", form)
            regexp13 = re.search("[kKCc]asss?ette", form)
            regexp14 = re.search("^VHS", form)
            regexp15 = re.search("^Noten", form)
            regexp16 = re.search("^Losebl", form)
            regexp17 = re.search("^Film\s?\[", form)
            regexp18 = re.search("\d\smin", form)
            regexp19 = re.search("S\.\s\d+\s?-\s?\d+", form)

            if regexp1 or regexp2 or regexp3 or regexp4 or regexp5 or regexp6:
                format = "Buch"
            elif regexp7:
                format = "DVD"
            elif regexp8:
                format = "Blu-ray"
            elif regexp9 or regexp10 or regexp11:
                format = "Videodatei"
            elif regexp12:
                format = "CD"
            elif regexp13 or regexp14:
                format = "Videokassette"
            elif regexp15:
                format = "Noten"
            elif regexp16:
                format = "Loseblattsammlung"
            elif regexp17 or regexp18:
                format = "Film"
            elif regexp19:
                format = "Aufsatz"
            else:
                if form != "":
                    format = "Buch"
                    print("Format nicht erkannt: %s (Default = Buch)" % form)

        # 1. Urheber
        if f100a == "":
            f100a = get_subfield("100", "a")
            if f100a != "":
                f100.append("a")
                f100.append(f100a)
                f100e = get_subfield("100", "b")
                if f100e != "":
                    f100e = f100e.split("]")
                    for role in f100e[:-1]:  # Slice :-1 damit das letzte ] ausgelassen wird
                        role = role + "]"
                        f100.append("e")
                        f100.append(role)

        # Haupttitel
        if f245a == "":
            f245a = get_field("331")

        # Erscheinungsort
        if f260a == "":
            f260a = get_field("410")

        # Verlag
        if f260b == "":
            f260b = get_field("412")

        # Erscheinungsjahr
        if f260c == "":
            f260c = get_field("425")

        # physische Beschreibung
        if f300a == "":
            f300 = get_field("433")
            # 335 S. : zahlr. Ill. ; 32 cm
            regexp1 = re.search("(.*)\s?:\s(.*);\s(.*)", f300)
            # 289 S.: Zahlr. Ill.
            regexp2 = re.search("(.*)\s?:\s(.*)", f300)
            # 106 S. ; 21 cm
            regexp3 = re.search("(.*)\s?;\s(.*)", f300)

            if regexp1:
                f300a, f300b, f300c = regexp1.groups()
                f300a = f300a + " : "
                f300b = f300b + " ; "
            elif regexp2:
                f300a, f300b = regexp2.groups()
                f300a = f300a + " : "
                f300c = ""
            elif regexp3:
                f300a, f300c = regexp3.groups()
                f300a + " ; "
                f300b = ""
            else:
                f300a = f300
                f300b = ""
                f300c = ""

        # Schlagwörter
        f650a = get_subfield("710", "a")
        if f650a != "":
            subjects.append(f650a)

        # weitere Personen
        regexp = re.search('tag="1\d\d"', field)
        if regexp:
            for i in range(
                    101, 197
            ):  # überprüfen, ob ein Personenfeld vorliegt, damit die Schleife für die Personenfelder nicht bei jedem Feld durchlaufen wird
                f700a = get_subfield(i, "a")
                if f700a != "":
                    f700.append("a")
                    f700.append(f700a)
                    f700e = get_subfield(i, "b")
                    if f700e != "":
                        f700e = f700e.split("]")
                        for role in f700e[:-1]:  # Slice :-1 damit das letzte ] ausgelassen wird
                            role = role + "]"
                            f700.append("e")
                            f700.append(role)
                    persons.append(f700)
                    f700 = []
                    break

    if format == "":
        format = "Buch"

    leader = get_leader(format=format)
    marcrecord.leader = leader

    marcrecord.add("001", data="finc-151-" + f001)

    f007 = get_field_007(format=format)
    marcrecord.add("007", data=f007)

    marcrecord.add("020", a=f020a)

    marcrecord.add("100", subfields=f100)

    marcrecord.add("245", a=f245a)

    publisher = ["a", "Hamburg : ", "b", f260b + ", ", "c", f260c]
    marcrecord.add("260", subfields=publisher)

    physicaldescription = ["a", f300a, "b", f300b, "c", f300c]
    marcrecord.add("300", subfields=physicaldescription)

    for f650a in subjects:
        marcrecord.add("650", a=f650a)

    for f700 in persons:
        marcrecord.add("700", subfields=f700)

    f935b = get_field_935b(format=format)
    f935c = get_field_935c(format=format)
    marcrecord.add("935", a="vkfilm", b=f935b, c=f935c)

    collections = ["a", f001, "b", "151", "c", "sid-151-col-filmakademiebawue"]
    marcrecord.add("980", subfields=collections)

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()



#!/usr/bin/env python
# coding: utf-8
"""
Notes:

* XML contains entities: "... Bedeutung und Gef&#xE4;hrdung der Sinne ..."
"""

from __future__ import print_function

import base64
import io
import re
import sys
from builtins import *

import xmltodict

import marcx

try:
    basestring
except NameError:
    basestring = str

lang_map = {"": "ger", "de": "ger", "deu": "ger"}

# Default input and output.
inputfilename = "73_input_datacite.xml"
outputfilename = "73_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "r", encoding='utf-8')
outputfile = io.open(outputfilename, "wb")
xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for i, xmlrecord in enumerate(xmlrecords["Records"]["Record"]):
    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Leader
    marcrecord.leader = "     nab  22        4500"

    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    f001 = f001.encode("utf-8")
    f001 = base64.b64encode(f001)
    f001 = f001.decode("ascii").rstrip("=")
    marcrecord.add("001", data="finc-73-" + f001)

    # 007
    marcrecord.add("007", data="cr")

    # ISSN
    try:
        identifiers = xmlrecord.get("metadata", {}).get("dcite:resource",
                                                        {}).get("dcite:relatedIdentifiers",
                                                                {}).get("dcite:relatedIdentifier", [])
        for identifier in identifiers:
            if identifier["@relatedIdentifierType"] == "ISSN":
                f022a = identifier["#text"]
                marcrecord.add("022", a=f022a)
                break
    except Exception as exc:
        pass

    # Sprache
    try:
        language = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:language", "")
        f041a = lang_map.get(language, "")
        if f041a != "":
            pass
            marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % f041a)
            marcrecord.add("041", a=f041a)
        else:
            print("Die Sprache %s fehlt in der Lang_Map!" % language)
    except Exception as exc:
        print('[language] %s' % exc, file=sys.stderr)

    # 1. Urheber
    try:
        f100a = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:creators",
                                                                            {}).get("dcite:creator", "")
    except Exception as exc:
        f100a = ""
        print('[100] %s' % type(f100a))
    if isinstance(f100a, list):
        f100a = f100a[0]
    elif isinstance(f100a, dict):
        f100a = f100a["dcite:creatorName"]["#text"]
    elif isinstance(f100a, str):
        print(f100a)
    marcrecord.add("100", a=f100a)

    # Haupttitel, Titelzusatz, Verantwortlichenangabe
    try:
        f245 = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:titles", {}).get("dcite:title", "")

        if "MEDIENwissenschaft: Rezensionen | Reviews" in f245:  # überspringt Gesamtaufnahmen der Zeitschriftehefte
            continue

        if not isinstance(f245, basestring):
            print("245 is not a string: %s (%s)" % (f245, type(f245)), file=sys.stderr)
            continue

        f245 = f245.split(":")
        f245a = f245[0]
        f245a = f245a.strip(" ")
        if len(f245) > 1:
            f245a += " - " + f245[1].strip(" ")
        if len(f245) > 2:
            f245b = ""
            for titlepart in f245[2:]:
                titlepart = titlepart.strip(" ")
                f245b += titlepart + " : "
        else:
            f245b = ""
        marcrecord.add("245", a=f245a, b=f245b.rstrip(" : "))
    except Exception as exc:
        print("[245] %s: %s" % (f245, exc), file=sys.stderr)
        continue

    # Erscheinungsvermerk
    f260c = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:publicationYear", "")
    publisher = ["a", "Marburg", "b", " : " + u"Schüren Verlag, ", "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # Rechtehinweis
    try:
        f500a = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:rightsList", "")
        if f500a:
            f500a = f500a.get("dcite:rights")
            f500a = f500a[0]
            marcrecord.add("500", a=f500a)
    except Exception as exc:
        print('[500]', file=sys.stderr)
        raise

    # Schlagwort
    try:
        f650a = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:subjects", [])
        if f650a:
            f650a = f650a.get("dcite:subject")
            if isinstance(f650a, list):
                for subject in f650a:
                    if subject:
                        marcrecord.add("650", a=subject)
            else:
                marcrecord.add("650", a=f650a)
    except Exception as exc:
        print('[650]', file=sys.stderr)
        raise

    # übergeordnete Ressource
    f773g = xmlrecord.get("metadata", {}).get("dcite:resource", {}).get("dcite:publicationYear", "")
    marcrecord.add("773", g="(" + f773g + ")", t=u"MEDIENwissenschaft: Rezensionen | Reviews")

    # Link zur Ressource
    try:
        identifiers = xmlrecord.get("metadata", {}).get("dcite:resource",
                                                        {}).get("dcite:relatedIdentifiers",
                                                                {}).get("dcite:relatedIdentifier", [])
        for identifier in identifiers:
            if identifier["@relatedIdentifierType"] == "URL":
                f856u = identifier["#text"]
                marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)
                break
    except Exception as exc:
        print('[856]', file=sys.stderr)
        raise

    # Medienform
    marcrecord.add("935", b="cofz")

    # Kollektion
    marcrecord.add("980", a=f001, b="73", c="sid-73-col-medienw")

    try:
        outputfile.write(marcrecord.as_marc())
    except UnicodeDecodeError as exc:
        print("#%08d failed: %s" % (i, marcrecord["001"]), file=sys.stderr)
        raise

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301
# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
JOVE refs #11308, #14062, #14962.

Config
------

[jove]

url = https://www.jove.com/api/articles/v0/articlefeed.php?marc=1&journal=1

"""

import datetime

import luigi
import requests

from gluish.utils import shellout
from siskin.task import DefaultTask


class JoveTask(DefaultTask):
    TAG = "143"


class JoveMARC(JoveTask):
    """
    Turn MRK MARC (https://www.loc.gov/marc/makrbrkr.html) into MARC XML via
    experimental marctexttoxml (https://git.io/vdCUN).
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("""curl -sL --fail "{url}" | marctexttoxml | xmllint --format - > {output}""",
                          url=self.config.get("jove", "url"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class JoveFincMARC(JoveTask):
    """
    Convert JOVE to fincmarc.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return JoveMARC(date=self.date)

    def run(self):
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets("143/143_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



#!/usr/bin/env python3
# coding: utf-8

# Quelle: Institut Ägyptologie
# SID: 70
# Tickets: #5246, #13232, #14359
# technicalCollectionID: sid-70-col-aegyptologie
# Task: egyptology.py

from __future__ import print_function

import io
import os
import re
import sqlite3
import sys
from builtins import *

import marcx
from siskin.utils import marc_build_imprint

# XXX: https://stackoverflow.com/q/3828723
reload(sys)
sys.setdefaultencoding('utf8')

lang_map = {
    "de": "ger",
    "en": "eng",
    "fr": "fre",
    "ru": "rus",
    "it": "ita",
    "es": "spa",
    "af": "afr",
    "cs": "csb",
    "ar": "ara",
    "hu": "hung"
}

inputfilename = "70_input.ctv6"
outputfilename = "70_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

sqlitecon = sqlite3.connect(inputfilename)
outputfile = io.open(outputfilename, "wb")

query = """
    SELECT
        Reference.ID as id,
        Title as title,
        Subtitle as subtitle,
        ISBN as isbn,
        LanguageCode as language,
        PageCount as pages,
        PlaceOfPublication as place,
        Publisher.Name as publisher,
        Year as year,
        ReferenceType as format,
        Volume as volume,
        Periodical.Name as jtitle,
        GROUP_CONCAT(Author , "; " ) As authors,
        GROUP_CONCAT(Editor , "; " ) As editors,
        Signatur as callnumber,
        SeriesTitle.Name as series,
        PageRange as range
    FROM
        Reference
    LEFT JOIN
        Periodical ON (Periodical.ID = Reference.PeriodicalID)
    LEFT JOIN
        ReferencePublisher ON (ReferencePublisher.ReferenceID = Reference.ID)
    LEFT JOIN
        Publisher ON (Publisher.ID = ReferencePublisher.PublisherID)
    LEFT JOIN
        ReferenceAuthor ON (ReferenceAuthor.ReferenceID = Reference.ID)
    LEFT JOIN
        (SELECT
            ID as AuthorID,
            substr(COALESCE(LastName, '') || ', ' || COALESCE(FirstName, '') || ' ' || COALESCE(MiddleName, ''), 0) as Author
        FROM
            Person) ON (AuthorID = ReferenceAuthor.PersonID)
    LEFT JOIN
        ReferenceEditor ON (ReferenceEditor.ReferenceID = Reference.ID)
    LEFT JOIN
        (SELECT
            ID as EditorID,
            substr(COALESCE(LastName, '') || ', ' || COALESCE(FirstName, '') || ' ' || COALESCE(MiddleName, ''), 0) as Editor
        FROM
            Person) ON (EditorID = ReferenceEditor.PersonID)
    LEFT JOIN
        (SELECT
            GROUP_CONCAT(CallNumber , "; " ) AS Signatur,
            ReferenceID
        FROM
            Location
        GROUP BY
            ReferenceID) as Location2 ON (Location2.ReferenceID = Reference.ID)
    LEFT JOIN
        SeriesTitle ON (SeriesTitle.ID = Reference.SeriesTitleID)
    WHERE
        Title NOT NULL
    GROUP BY
        Reference.ID
    """

sqlite = sqlitecon.cursor()
sqlite.execute(query)

for i, record in enumerate(sqlite):

    # Cleanup None values
    record = [r if r is not None else '' for r in record]

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    author_doublets = []

    # Formatzuweisung
    format = record[9]
    if format == "JournalArticle" or format == "NewspaperArticle":
        leader = "     naa  22        4500"
        f935c = "text"
    elif format == "Thesis":
        leader = "     cam  22        4500"
        f935c = "hs"
    else:
        leader = "     cam  22        4500"
        f935c = "lo"

    f007 = "tu"
    f935b = "druck"

    # Identifikator
    f001 = str(i)
    marcrecord.add("001", data="finc-70-" + f001)

    # Leader
    marcrecord.leader = leader

    # Format 007
    marcrecord.add("007", data=f007)

    # ISBN
    f020a = record[3]
    if f020a:
        match = re.search("([0-9xX-]{10,17})", f020a)
        if match:
            marcrecord.add("020", a=f020a)

    # ISSN
    f022a = record[3]
    if f022a:
        match = re.search("([0-9xX-]{9,9})", f022a)
        if match:
            marcrecord.add("022", a=f022a)

    # Sprache
    language = record[4]
    f041a = lang_map.get(language, "")
    if f041a != "":
        marcrecord.add("008", data="130227uu20uuuuuuxx uuu %s  c" % f041a)
        marcrecord.add("041", a=f041a)
    else:
        print("Die Sprache %s fehlt in der Lang_Map!" % language)

    # 1. Urheber
    persons = record[12]
    if persons:
        persons = persons.split(";")
        f100a = persons[0]
        f100a = f100a.strip()
        marcrecord.add("100", a=f100a)
        author_doublets.append(f100a)

    # Titel
    f245a = record[1]
    f245b = record[2]
    marcrecord.add("245", a=f245a, b=f245b)

    # Erscheinungsvermerk
    f260a = record[6]
    f260b = record[7]
    f260c = record[8]
    subfields = marc_build_imprint(f260a, f260b, f260c)
    marcrecord.add("260", subfields=subfields)

    # Umfangsangabe
    f300a = record[5]
    match = re.search("\<c\>(\d+)\</c\>", f300a)
    if match:
        f300a = match.group(1)
        f300a = f300a + " S."
        marcrecord.add("300", a=f300a)

    # Schriftenreihe
    f490a = record[15]
    marcrecord.add("490", a=f490a)

    # weitere Urheber
    persons = record[12]
    if persons:
        persons = persons.split(";")
        for person in persons[1:]:
            f700a = person.strip()
            if f700a in author_doublets:
                continue
            marcrecord.add("700", a=f700a)
            author_doublets.append(f700a)

    # Herausgeber
    editors = record[13]
    if editors:
        editors = editors.split(";")
        editors = set(editors)  # wegen der Dopplungen
        for editor in editors:
            f700a = editor.strip()
            marcrecord.add("700", a=f700a)

    # Verweis auf Zeitschrift
    if format == "JournalArticle" or format == "NewspaperArticle":
        f773t = record[11]
        year = record[8]
        volume = record[10]
        pages = record[16]
        match = re.search("\<os\>(\d+-\d+)\</os\>", pages)

        if match:
            pages = match.group(1)
            pages = ", S. " + pages
        else:
            pages = ""

        if f773t and (volume or year or pages):
            f773g = volume + "(" + year + ")" + pages
        else:
            f773g = ""

        marcrecord.add("773", t=f773t, g=f773g)

    # SWB-Format
    marcrecord.add("935", b=f935b, c=f935c)

    # Link zur Bestandsinfo
    callnumber = record[14]
    if not callnumber:
        callnumber = "nicht verfügbar"
    marcrecord.add(
        "856",
        q="text/html",
        _3="Link zur Bestandsinformation",
        u="http://www.gko.uni-leipzig.de/de/aegyptologisches-institut/bibliothek/informationen.html",
        z="Bestand der Bibliothek des Ägyptologischen Institus, bitte informieren Sie sich vor Ort. Signatur: " +
        callnumber)

    # Kollektion
    collections = ["a", f001, "b", "70", "c", "sid-70-col-aegyptologie"]
    marcrecord.add("980", subfields=collections)

    outputfile.write(marcrecord.as_marc())

sqlitecon.close()
outputfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401,E0202

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Entry points should replace scripts in the future. Currently we use a mix of
bash and Python for executables and console_scripts - with bash not being cross
platform anyway.

TODO(miku): Consider moving to a single entry point, e.g. "siskin" with
subcommands implemented by separate scripts.
"""

from __future__ import print_function

import sys

from luigi.task import Register

from siskin import __version__
from siskin.benchmark import green, yellow
from siskin.sources import *
from siskin.workflows import *


def main():
    print("siskin %s\n\n" % __version__)
    task_names = Register.task_names()
    print('{0} tasks found\n'.format(len(task_names)))

    for name in task_names:
        if name.islower():
            continue
        klass = Register.get_task_cls(name)
        doc = klass.__doc__ or yellow("@TODO: docs")
        print('{0} {1}\n'.format(green(name), doc))



# coding: utf-8
# pylint: disable=C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

#
# Adhoc
# =====
#

import datetime
import json
import tempfile

import luigi
import requests
import xlsxwriter

from gluish.format import TSV, Gzip
from gluish.utils import shellout
from siskin.sources.amsl import AMSLCollections, AMSLService
from siskin.sources.crossref import (CrossrefCollections, CrossrefCollectionsCount, CrossrefCollectionsDifference,
                                     CrossrefExport)
from siskin.sources.degruyter import DegruyterExport
from siskin.sources.doaj import DOAJExport
from siskin.sources.elsevierjournals import ElsevierJournalsExport
from siskin.sources.highwire import HighwireExport
from siskin.sources.jstor import JstorExport
from siskin.task import DefaultTask
from siskin.utils import SetEncoder, load_set_from_target


class AdhocTask(DefaultTask):
    """
    Base task for throwaway tasks.
    """
    TAG = "adhoc"


class AdhocFormetaSamples(AdhocTask):
    """
    Formeta test samples.
    """

    def requires(self):
        return [
            CrossrefExport(format='formeta'),
            DegruyterExport(format='formeta'),
            DOAJExport(format='formeta'),
            ElsevierJournalsExport(format='formeta'),
            HighwireExport(format='formeta'),
            JstorExport(format='formeta'),
        ]

    def run(self):
        with self.output().open('w') as output:
            for target in self.input():
                output.write_tsv(target.path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class OADOIDatasetStatusByDOI(AdhocTask):
    """
    Download oadoi.org dataset.
    See release notes at: https://docs.google.com/document/d/1Whfe26oyjTedeW1GGWkq3NADgDbL2R2eXqrJCWS8vcc/edit#

    > These datasets are for non-commercial use. For commercial use, ask us
      about our Support Level Agreement (team@impactstory.org). For more
      information about oaDOI, see http://oadoi.org.
    """

    def run(self):
        url = "https://s3-us-west-2.amazonaws.com/oadoi-datasets/oa_status_by_doi.csv.gz"
        output = shellout("""wget -O "{output}" "{url}" """, url=url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="csv.gz"), format=Gzip)


class Issue7049(AdhocTask):
    """
    Prepare a few bits about collections, refs #7049.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'amsl': AMSLCollections(date=self.date),
            'crossref': CrossrefCollections(date=self.date),
        }

    def run(self):
        amsl = load_set_from_target(self.input().get('amsl'))
        crossref = load_set_from_target(self.input().get('crossref'))

        with self.output().open('w') as output:
            stats = {
                'amsl': amsl,
                'crossref': crossref,
                'amsl_only': amsl - crossref,
                'crossref_only': crossref - amsl,
                'both': amsl & crossref,
            }
            output.write(json.dumps(stats, cls=SetEncoder))

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="json"))


class Issue7049ExportExcel(AdhocTask):
    """
    Create a simple Excel file, with one sheet per kind.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'issue7049': Issue7049(date=self.date),
            'counts': CrossrefCollectionsCount(date=self.date),
        }

    def run(self):
        with self.input().get('counts').open() as handle:
            counts = json.load(handle)

        with self.input().get('issue7049').open() as handle:
            doc = json.load(handle)

        _, stopover = tempfile.mkstemp(prefix='siskin-')
        workbook = xlsxwriter.Workbook(stopover)

        worksheet = workbook.add_worksheet(name='Overview')

        keys = doc.keys()

        worksheet.write(0, 0, "#7049")
        worksheet.write(1, 0, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

        for i, key in enumerate(keys, start=3):
            worksheet.write(i, 1, key)
            worksheet.write(i, 2, len(doc[key]))

        for _, key in enumerate(keys):
            worksheet = workbook.add_worksheet(name=key)
            for j, item in enumerate(sorted(doc[key])):
                worksheet.write(j, 0, counts.get(item, 'NA'))
                worksheet.write(j, 1, item)

        workbook.close()
        luigi.LocalTarget(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xlsx"))


class K10Matches(AdhocTask):
    """
    How many of the ISSN in K10Plus are indexed in finc?

    CSV list: https://is.gd/AW3bCB
    """

    ai = luigi.Parameter(default="http://localhost:8983/solr/biblio")
    finc = luigi.Parameter(default="http://localhost:8983/solr/biblio")

    def run(self):
        r = requests.get("https://is.gd/AW3bCB")
        with self.output().open("w") as output:
            for line in r.text.split('\n'):
                try:
                    issn, count = line.split(',')

                    results = {}

                    results['ai'] = requests.get("%s/select?q=issn:%s&rows=0&wt=json" % (self.ai, issn))
                    if results['ai'].status_code != 200:
                        raise RuntimeError("ai reponded with %s" % rr.status_code)

                    results['finc'] = requests.get("%s/select?q=issn:%s&rows=0&wt=json" % (self.finc, issn))
                    if results['finc'].status_code != 200:
                        raise RuntimeError("finc reponded with %s" % rr.status_code)

                    output.write_tsv(
                        issn,
                        count,
                        str(results['ai'].json().get("response").get("numFound")),
                        str(results['finc'].json().get("response").get("numFound")),
                    )

                except Exception as exc:
                    self.logger.debug(exc)

    def output(self):
        return luigi.LocalTarget(path=self.path(digest=True), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Elsevier jounrals. Refs. #6975.

Configuration keys:

[elsevierjournals]

ftp-host = sftp://host.name
ftp-username = username
ftp-password = password
ftp-path = /
ftp-pattern = *

backlog-dir = /path/to/dir

"""

import datetime
import os
import tempfile
from builtins import str

import luigi

from gluish.common import Executable
from gluish.format import TSV, Gzip
from gluish.utils import shellout
from siskin.benchmark import timed
from siskin.common import FTPMirror
from siskin.sources.amsl import AMSLFilterConfig
from siskin.task import DefaultTask
from siskin.utils import iterfiles


class ElsevierJournalsTask(DefaultTask):
    """ Elsevier journals base. """
    TAG = '85'


class ElsevierJournalsBacklogIntermediateSchema(ElsevierJournalsTask):
    """
    Convert backlog to intermediate schema.
    """

    def run(self):
        directory = self.config.get('elsevierjournals', 'backlog-dir')
        _, output = tempfile.mkstemp(prefix='siskin-')
        for path in sorted(iterfiles(directory, fun=lambda p: p.endswith('.tar'))):
            shellout("span-import -i elsevier-tar {input} | pigz -c >> {output}", input=path, output=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class ElsevierJournalsPaths(ElsevierJournalsTask):
    """
    Sync.
    """
    date = luigi.DateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20, significant=False, description='timeout in seconds')

    def requires(self):
        return FTPMirror(host=self.config.get('elsevierjournals', 'ftp-host'),
                         username=self.config.get('elsevierjournals', 'ftp-username'),
                         password=self.config.get('elsevierjournals', 'ftp-password'),
                         pattern=self.config.get('elsevierjournals', 'ftp-pattern'),
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    @timed
    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class ElsevierJournalsUpdatesIntermediateSchema(ElsevierJournalsTask):
    """
    Intermediate schema from updates.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return ElsevierJournalsPaths(date=self.date)

    @timed
    def run(self):
        _, output = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in sorted(handle.iter_tsv(cols=('path', ))):
                if not str(row.path).endswith('.tar'):
                    continue
                shellout("span-import -i elsevier-tar {input} | pigz -c >> {output}", input=row.path, output=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class ElsevierJournalsIntermediateSchema(ElsevierJournalsTask):
    """ Combine backlog and updates. """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return [ElsevierJournalsBacklogIntermediateSchema(), ElsevierJournalsUpdatesIntermediateSchema(date=self.date)]

    @timed
    def run(self):
        _, output = tempfile.mkstemp(prefix='siskin-')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path, output=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class ElsevierJournalsExport(ElsevierJournalsTask):
    """
    Tag with ISILs, then export to various formats.
    """
    date = luigi.DateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='solr5vu3', description='export format')

    def requires(self):
        return {
            'file': ElsevierJournalsIntermediateSchema(date=self.date),
            'config': AMSLFilterConfig(date=self.date),
        }

    def run(self):
        output = shellout("span-tag -c {config} <(unpigz -c {input}) | pigz -c > {output}",
                          config=self.input().get('config').path,
                          input=self.input().get('file').path)
        output = shellout("span-export -o {format} <(unpigz -c {input}) | pigz -c > {output}",
                          format=self.format,
                          input=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        extensions = {
            'solr5vu3': 'ldj.gz',
            'formeta': 'form.gz',
        }
        return luigi.LocalTarget(path=self.path(ext=extensions.get(self.format, 'gz')))


class ElsevierJournalsDOIList(ElsevierJournalsTask):
    """
    A list of Elsevier journals DOIs.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'input': ElsevierJournalsIntermediateSchema(date=self.date),
            'jq': Executable(name='jq', message='https://github.com/stedolan/jq')
        }

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        # process substitution sometimes results in a broken pipe, so extract
        # beforehand
        output = shellout("unpigz -c {input} > {output}", input=self.input().get('input').path)
        shellout("""jq -r '.doi?' {input} | grep -o "10.*" 2> /dev/null | LC_ALL=C sort -S50% > {output} """,
                 input=output,
                 output=stopover)
        os.remove(output)
        output = shellout("""sort -S50% -u {input} > {output} """, input=stopover)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class ElsevierJournalsISSNList(ElsevierJournalsTask):
    """
    A list of Elsevier journals DOIs.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'input': ElsevierJournalsIntermediateSchema(date=self.date),
            'jq': Executable(name='jq', message='https://github.com/stedolan/jq')
        }

    @timed
    def run(self):
        _, output = tempfile.mkstemp(prefix='siskin-')
        shellout("""jq -c -r '.["rft.issn"][]?' <(unpigz -c {input}) >> {output} """,
                 input=self.input().get('input').path,
                 output=output)
        shellout("""jq -c -r '.["rft.eissn"][]?' <(unpigz -c {input}) >> {output} """,
                 input=self.input().get('input').path,
                 output=output)
        output = shellout("""sort -u {input} > {output} """, input=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)
